# -------------------------------------------------
# Atari ST interleaved bitmap to Amiga Planar 1.0
# Ruby script written by Franck "hitchhikr" Charlet (hhhikr@gmail.com).
# -------------------------------------------------

# -------------------------------------------------
# Program entry point
# -------------------------------------------------

    print "PIC to RAW v1.0\n"
    print "Written by Franck Charlet.\n"

# -------------------------------------------------
# Check command line arguments
    if ARGV.length < 4
        print "\nUsage: " + $0 + " <filename.pic> <width> <height> <palette file> [filename.raw]\n"
        exit
    end

    source_name = ARGV[0]

    width = Integer(ARGV[1])
    height = Integer(ARGV[2])
    palette_name = ARGV[3]

    # Create the output name if necessary
    if(ARGV.length < 5)
        dest_name = File::dirname(source_name) + File::ALT_SEPARATOR + File.basename(source_name, File.extname(source_name)) + ".raw"
        dest_palette_name = File::dirname(source_name) + File::ALT_SEPARATOR + File.basename(source_name, File.extname(source_name)) + ".raw.pal"
    else
        dest_name = ARGV[4]
        dest_palette_name = File::dirname(source_name) + File::ALT_SEPARATOR + File.basename(source_name, File.extname(source_name)) + ".raw.pal"
    end
    
    print "\nConverting " + source_name + " (" + String(width) + "x" + String(height) + ") to " + dest_name + ".\n"

    fin = File.open(source_name, 'rb')

    amPlane1 = Array.new(((width / 8) / 2) * height)
    amPlane2 = Array.new(((width / 8) / 2) * height)
    amPlane3 = Array.new(((width / 8) / 2) * height)
    amPlane4 = Array.new(((width / 8) / 2) * height)

    i = 0
    x = 0
    y = 0

    # Process whole file
    while y < height do
        x = 0
        # Process 1 line
        while x < (((width / 2) / 4) / 2) do
            atplane1 = ((fin.read(1).unpack('C')[0] << 8) | fin.read(1).unpack('C')[0]) & 0xffff
            atplane2 = ((fin.read(1).unpack('C')[0] << 8) | fin.read(1).unpack('C')[0]) & 0xffff
            atplane3 = ((fin.read(1).unpack('C')[0] << 8) | fin.read(1).unpack('C')[0]) & 0xffff
            atplane4 = ((fin.read(1).unpack('C')[0] << 8) | fin.read(1).unpack('C')[0]) & 0xffff
            amPlane1[x + (y * (width / 8))] = atplane1
            amPlane2[x + (y * (width / 8))] = atplane2
            amPlane3[x + (y * (width / 8))] = atplane3
            amPlane4[x + (y * (width / 8))] = atplane4
            x += 1
            i += 8
        end
        y += 1
    end

    processed = i
    original = fin.stat.size

    fin.close()

    fout = File.open(dest_palette_name, 'wb')
    if palette_name != ""
        fin = File.open(palette_name, 'rb')
        i = 0
        print "FUCK"
        while i < 16 * 2 do
            fout.putc(fin.read(1).unpack('C')[0] << 1)
            i += 1
        end
        fin.close()
    end

    fout = File.open(dest_name, 'wb')

    y = 0
    while y < height do
        x = 0
        while x < (((width / 2) / 4) / 2) do
            fout.putc(amPlane1[x + (y * (width / 8))] >> 8)
            fout.putc(amPlane1[x + (y * (width / 8))])
            x += 1
        end
        y += 1
    end    

    y = 0
    while y < height do
        x = 0
        while x < (((width / 2) / 4) / 2) do
            fout.putc(amPlane2[x + (y * (width / 8))] >> 8)
            fout.putc(amPlane2[x + (y * (width / 8))])
            x += 1
        end
        y += 1
    end    

    y = 0
    while y < height do
        x = 0
        while x < (((width / 2) / 4) / 2) do
            fout.putc(amPlane3[x + (y * (width / 8))] >> 8)
            fout.putc(amPlane3[x + (y * (width / 8))])
            x += 1
        end
        y += 1
    end    

    y = 0
    while y < height do
        x = 0
        while x < (((width / 2) / 4) / 2) do
            fout.putc(amPlane4[x + (y * (width / 8))] >> 8)
            fout.putc(amPlane4[x + (y * (width / 8))])
            x += 1
        end
        y += 1
    end    

    fout.close()

    print "Bytes processed: " + String(processed) + " bytes\n"
    if(processed < original)
        print "WARNING: Original file size was: " + String(original) + "\n"
    else
        print "Done.\n"
    end
